#include "Boss.h"

#define FINAL_BOSS_CELL_COUNT_ 4
#define CARRY_MINE_DAMAGE_ 5

u8 bossRedraw = TRUE;
u8 finalBossHealth = 0;

u16 plantPosition_ = 0;
u8 plantCell_ = 0;
u8 finalBossDirection_ = ENEMY_DIRECTION_DOWN_;

const u8 relativePositions_[FINAL_BOSS_CELL_COUNT_] = { 0 , 1, MAP_TILES_Y, MAP_TILES_Y + 1 };

void initBoss()
{
	plantCell_ = 0;
	plantPosition_ = 0;
	finalBossHealth = finalBossStartingHealth;
	
	// Randomly choose a direction from 0 to 3.
	finalBossDirection_ = cpct_rand() % 4;
}

void drawBossCells(u8 xScroll, u8 yScroll)
{
    if(bossRedraw)
    {
        u8 i = 0;
		u8 cell = 0;

		// Don't draw the boss after it is dead.
		if (currentLevel == BOSS_LEVEL)
		{
			if (finalBossHealth == 0)
			{
				bossRedraw = FALSE;
				return;
			}
		}

		// Draw each of the boss cells where I determine if the cells are actually on screen.
		for (i = 0; i < mapBossCellCount; i += 3)
		{
			cell = MAPA(mapBossCells[i], mapBossCells[i + 1]);

			// Enemy of type 0 is dead.
			if (mapBossCells[i + 2])
			{
				drawBossCell(mapBossCells[i] - xScroll, mapBossCells[i + 1] - yScroll, mapBossCells[i + 2], cell);
			}
		}

		bossRedraw = FALSE;
	}
}

void drawBossCell(i8 x, i8 y, u8 type, u8 cell)
{
	// Make sure the enemy is in bounds on the x axis.
	if (x >= 0 && x < SCREEN_TILES_X)
	{
		// Make sure the enemy is in bounds on the y axis.
		if (y >= 0 && y < SCREEN_TILES_Y)
		{
			u8* mapMemory = cpct_getScreenPtr(CPCT_VMEM_START, xPositions[x], yPositions[y]);

			// Only draw the boss cell if the cell is lit.
			if (cell & CELL_LIGHT_FLAG)
			{
				// Draw a boss sprite from the graphicset.	
				cpct_drawSprite(bossGraphics[type - 1], mapMemory, TILE_WIDTH_BYTES, TILE_HEIGHT);
			}
		}
	}
}

void logicBoss()
{
	// Run through the logic for all the boss if it is the boss' turn.
	if (gameTurnState == BOSS_TURN)
	{
		if (currentLevel == GROWING_PLANT_LEVEL)
		{
			if (plantCell_ < mapBossCellCount)
			{
				// Initialise the plantPosition for the current cell of the plant.
				if (!plantPosition_)
				{
					plantPosition_ = XY_TO_POSITION(mapBossCells[plantCell_], mapBossCells[plantCell_ + 1]);
				}

				plantPosition_ = nextPosition(plantPosition_);

				if (plantPosition_)
				{
					convertToPlant(mapCells[plantPosition_] & CELL_GFX_RANGE);
				}
				else
				{
					// This cell has been completed so move on to the next one.
					plantCell_ += 3;
				}
			}
		}
		else if (currentLevel == BOSS_LEVEL)
		{
			u8 i = 0;
			u8 j = 0;

			// For my convienience I am assuming that the boss is at mapBossCells index 0 to 11.
			// Map boss array structure is the following:
			// 0Q | 2X
			// 1R | 3T

			// bossCells
			// 0  = Qx
			// 1  = Qy
			// 3  = Rx
			// 4  = Ry
			// 6  = Sx
			// 7  = Sy
			// 9  = Tx
			// 10 = Ty

			// This is the position of the top left cell.
			u8 bossX = mapBossCells[0];
			u8 bossY = mapBossCells[1];
			u16 bossPosition = XY_TO_POSITION(bossX, bossY);

			// To get to position Q,R,S,T from a starting position of Q.
			u8 relativePosition[FINAL_BOSS_CELL_COUNT_] = { 0 , 1, MAP_TILES_Y, MAP_TILES_Y + 1 };

			// The amount to move the position in each direction by 2 (LEFT, UP, RIGHT, DOWN).
			// Down goes +2, Right goes +72 (2*MAP_TILES_Y), Up goes -2, Left goes -72 (2*MAP_TILES_Y).
			i8 potentialPositionMoves[DIRECTION_COUNT] = { 72, 2, -72, -2 };
			
			// The amount that the map x,y values might change for each direction by 2.
			i8 potentialMapXYChange[DIRECTION_COUNT] = { 2, 2, -2, -2 };

			// The index start value for adjusting the map cell values.
			// RIGHT, DOWN, LEFT, UP
			u8 mapCellIndex[DIRECTION_COUNT] = { 0, 1, 0, 1 };

			// Directions are determined by controlled random.
			u8 determinedDirection[DIRECTION_COUNT];

			// Start with the chosen boss direction.
			u8 randomDirection = finalBossDirection_;

			// Make sure the boss is alive.
			if (finalBossHealth > 0)
			{
				// Iterate through all the directions to fill the determinedDirections array.
				// Up/down becomes left/right, down/up becomes left/right.
				for (i = 0; i < DIRECTION_COUNT; i += 2)
				{
					if (randomDirection == ENEMY_DIRECTION_DOWN_ || randomDirection == ENEMY_DIRECTION_UP_)
					{
						// Down or up so we want left 2 or right 0.
						// 0 to 1 becomes 0 or 2.
						randomDirection = (cpct_rand() % 2) * 2;
					}
					else
					{
						// Left or right we want up 3 or down 1.
						// 0 to 1 becomes 0 or 2, + 1 to get 1 or 3.
						randomDirection = 1 + (cpct_rand() % 2) * 2;
					}

					determinedDirection[i] = randomDirection;

					// Add two to get the next direction (opposite of current direction: left->right, up->down).
					randomDirection += 2;

					if (randomDirection > DIRECTION_COUNT)
					{
						randomDirection -= DIRECTION_COUNT;
					}

					determinedDirection[i + 1] = randomDirection;
				}

				for (i = 0; i < DIRECTION_COUNT; ++i)
				{
					u8 directionBlocked = FALSE;

					// Determine a direction and adjust test position accordingly.
					u16 testPosition = bossPosition;

					// Choose the next controlled random direction.
					finalBossDirection_ = determinedDirection[i];

					// Adjust the test position for that direction.
					testPosition += potentialPositionMoves[finalBossDirection_];

					// Test if that position is blocked.
					for (j = 0; j < FINAL_BOSS_CELL_COUNT_; ++j)
					{
						// Test if there something is in the cell that is not a carry mine.
						// Don't need to worry about map bounds because the boss is an arena that is always within map bounds.
						// 55 = CELL_ITEM_FLAG + ITEM_CARRY_MINE, 63 is the bottom 6 of 8 bits 0b00111111.
						if ((mapCells[testPosition + relativePosition[j]] & CELL_GFX_RANGE) && ((mapCells[testPosition + relativePosition[j]] & 63) != 55))
						{
							directionBlocked = TRUE;
						}
					}

					if (!directionBlocked)
					{
						// Update each of the values in the mapBossCells array.
						for (j = mapCellIndex[finalBossDirection_]; j < 12; j += 3)
						{
							mapBossCells[j] += potentialMapXYChange[finalBossDirection_];
						}

						// Updates the map values for the boss (it is stored as a boss indicator interactable on the map).
						updateBossPositionInMap(bossPosition, testPosition);

						// Break out of the loop as the boss has moved.
						break;
					}
				}
			}
			else
			{
				for (i = 0; i < FINAL_BOSS_CELL_COUNT_; ++i)
				{
					u16 relativeBossPosition = bossPosition + relativePositions_[i];
					
					if (i == 0)
					{
						// Set the top left position to be a visible exit.
						// INTERACTABLE_CAVE_ENTRANCE_OPEN+CELL_INTERACTABLE_FLAG+CELL_LIGHT_FLAG = 7+64+128 = 199
						mapCells[relativeBossPosition] = 199;
					}
					else
					{						
						// Clear the boss indicator at the previous position that the boss was at.	
						mapCells[relativeBossPosition] = CELL_LIGHT_FLAG;
					}
				}
			}
		}

		// Reset to player turn after enemy logic completed.
		gameTurnState = PLAYER_TURN;
	}
}

void updateBossPositionInMap(u16 bossPosition, u16 newBossPosition)
{
	u8 i = 0;
	u16 playerPosition = XY_TO_POSITION(player.object.position.x, player.object.position.y);

	for (i = 0; i < FINAL_BOSS_CELL_COUNT_; ++i)
	{
		u16 newBossRelativePosition = newBossPosition + relativePositions_[i];

		// Clear the boss indicator at the previous position that the boss has been at.
		mapCells[bossPosition + relativePositions_[i]] = 0;

		// Check to see if the new position has a carry mine on it.
		// 55 = CELL_ITEM_FLAG + ITEM_CARRY_MINE, 63 is the bottom 6 of 8 bits 0b00111111.
		if ((mapCells[newBossRelativePosition] & 63) == 55)
		{
			// The boss loses CARRY_MINE_DAMAGE_ health for every mine it is standing on.
			if (finalBossHealth < CARRY_MINE_DAMAGE_)
			{
				finalBossHealth = 0;
			}
			else
			{
				finalBossHealth -= CARRY_MINE_DAMAGE_;
			}
		}

		// Update the boss indicator at the new position that the boss is at. (Light is on deliberately).
		mapCells[newBossRelativePosition] = 222;

		// Player loses health if landed on by the boss but it isn't always fatal.
		if (playerPosition == newBossRelativePosition)
		{
			if (player.health < bossMoveDamage)
			{
				player.health = 0;
			}
			else
			{
				player.health -= bossMoveDamage;
			}
		}
	}

	// 128 + 30 + 64 = CELL_LIGHT_FLAG + CELL_LIGHT_FLAG + CELL_INTERACTABLE_FLAG + INTERACTABLE_BOSS_INDICATOR
	// 192 + 30 = 202+20 = 222
}

void convertToPlant(u8 cell)
{
	// I am deliberately ignoring the light flag because it is easier and then the plant grows "in the dark".
	// TODO player position and cell.
	if (cell == 13)
	{
		mapCells[plantPosition_] = CELL_INTERACTABLE_FLAG | INTERACTABLE_GROWING_PLANT_1;
	}
	else if (cell == 14)
	{
		mapCells[plantPosition_] = CELL_INTERACTABLE_FLAG | INTERACTABLE_GROWING_PLANT_2;
	}
	else if (cell == 15)
	{
		mapCells[plantPosition_] = CELL_INTERACTABLE_FLAG | INTERACTABLE_GROWING_PLANT_3;
	}
}

u16 nextPosition(u16 position)
{
	// Take the position and test north.
	if (isPlant(mapCells[position - 1]))
	{
		return position - 1;
	}

	// Test east.
	if (isPlant(mapCells[position + MAP_TILES_Y]))
	{
		return position + MAP_TILES_Y;
	}

	// Test south.
	if (isPlant(mapCells[position + 1]))
	{
		return position + 1;
	}
	
	// Test west.
	if (isPlant(mapCells[position - MAP_TILES_Y]))
	{
		return position - MAP_TILES_Y;
	}

	// No valid position, end of the line.
	// Technically there could be a plant at position 0 but the map will not have anything there and this way we can have false as 0.
	return 0;
}

u8 isPlant(u8 cell)
{
	// Only need the graphic.
	cell &= CELL_GFX_RANGE;

	// plants live have a graphics type of 12,13 and 14.
	return cell >= 13 && cell <= 15;
}