#ifndef ENEMY_H
#define ENEMY_H

#include <cpctelera.h>
#include "Cell.h"
#include "GameState.h"
#include "GameTypes.h"
#include "Player.h"
#include "Utility.h"
#include "sprites/EnemySprites.h"

#define ENEMY_GUARD 1
#define ENEMY_MOTH 2
#define ENEMY_LIGHT_ATTACKER 3
#define ENEMY_DARK_MINE 4

// Assume for a maximum of 3 bits for the graphic for now.
#define ENEMY_GFX_RANGE 0x07

// Enemy health flag, bitshift left twice to deplete enemy health.
#define ENEMY_HEALTH_FLAG 64

// The enemy direction bits (2^3=8, 2^4=16) are used to retrieve the enemy direction from the enemy state variable.
#define ENEMY_DIRECTION_BITS 24

// Indicates the enemy has been blocked by going in the specified direction.
#define ENEMY_BLOCKED_FLAG 32

// The enemy entities on the map.
extern u8 mapEnemies[MAX_ENEMY_ELEMENT_COUNT];

// The number of enemies on the map.
extern u16 mapEnemyCount;

// Determines when the enemy redraw should happen.
extern u8 enemyRedraw;

// Setup the health state for enemies.
void initEnemies();

// Draws the enemies on the screen.
void drawEnemies(u8 xScroll, u8 yScroll);

// Draw an individual enemy on the screen.
void drawEnemy(i8 x, i8 y, u8 enemyType, u8 cell);

// Runs all the logic for the enemies.
void logicEnemies();

// Attacks the player at player position.
void attackPlayer();

// Score values for enemies on killing them.
u8 enemyScore(u8 type);

// Rusn the logic for a standard enemy attacking the player.
u8 enemyAttackLogic(u8 x, u8 y, u8 enemyType);

#endif