#include "GameState.h"

u16 gameScore = 0;
u16 levelScore = 0;
u8 currentLevel = 255;
u8 levelEnded = LEVEL_READY;
u8 playerTeleportingX = 255;
u8 playerTeleportingY = 255;
u8 playerExiting = FALSE;
u8 preLevelTransitionPositionX = 1;
u8 preLevelTransitionPositionY = 1;
u8 waterLevel = 0;
u8 gameTurnState = PLAYER_TURN;
u8* G_tile_floor_sprite = NULL;
u8 oilFuel = 0;
u8 batteryFuel = 0;
u8 levelOilFuel = 0;
u8 levelBatteryFuel = 0;
u8* G_move_indicator = NULL;
u8 postMoveCell = 0;
u8 caveComplete[LEVEL_COUNT];
u8 arrowCount = 0;
u8 levelArrowCount = 0;
u8 darkMoveCount = 0;
u8 oilPlayerSteps = 0;
u8 batteryPlayerSteps = 0;

void resetGameSessionState()
{
	u8 i = 0;
    currentLevel = 0;
    levelEnded = LEVEL_READY;
	playerTeleportingX = 255;
	playerTeleportingY = 255;
	playerExiting = FALSE;
	preLevelTransitionPositionX = 1;
	preLevelTransitionPositionY = 1;
    waterLevel = FALSE;
	gameTurnState = PLAYER_TURN;
	G_tile_floor_sprite = NULL;
	oilFuel = 3;
	batteryFuel = 0;
	levelOilFuel = 0;
	levelBatteryFuel = 0;
	postMoveCell = 0;
	arrowCount = 0;
	levelArrowCount = 0;
	darkMoveCount = 0;
	gameScore = 0;
	oilPlayerSteps = 0;
	batteryPlayerSteps = 0;

    setupDifficulty();

	// Set all the caves as incomplete ready for the game.
	for (i = 0; i < LEVEL_COUNT; ++i)
	{
		caveComplete[i] = FALSE;
	}
}

void resetLevelSessionState()
{
	levelScore = 0;

	// Resets steps to 0 at the start of the level.
	oilPlayerSteps = 0;
	batteryPlayerSteps = 0;

	// Reset dark move count to the maximum number of dark moves available.
	setDarkMoves();
	darkMoveCount = maxDarkMoves;

	// Reset level fuel values to persistant game session values.
	levelOilFuel = oilFuel;
	levelBatteryFuel = batteryFuel;
	levelArrowCount = arrowCount;
}

void updateGameSessionState()
{
	gameScore += levelScore;

	// TODO for difficulty consider a base amount of fuel after each level.
	oilFuel = levelOilFuel;
	batteryFuel = levelBatteryFuel;
	arrowCount = levelArrowCount;
}

void updateFuelState()
{
	u8 playerLight = lightEquipped();

	// Fuel drain is turned off on easy mode.
	if (chosenDifficulty == EASY_DIFFICULTY)
	{
		return;
	}

	if (playerLight == ITEM_OIL_LAMP)
	{
		if (oilPlayerSteps < OIL_FUEL_DEPLETION_RATE)
		{
			++oilPlayerSteps;
		}
		else
		{
			// Light is only considered as equipped if they have fuel so we don't have to do an if test here.
			--levelOilFuel;

			// Deactivate the oil lamp if the player runs out of fuel.
			if (levelOilFuel == 0)
			{
				activateLightByType(playerLight, FALSE);
			}

			// Reset step count to 0.
			oilPlayerSteps = 0;
		}
	}
	else if (playerLight == ITEM_ELECTRIC_TORCH)
	{
		if (batteryPlayerSteps < BATTERY_FUEL_DEPLETION_RATE)
		{
			++batteryPlayerSteps;
		}
		else
		{
			// Light is only considered as equipped if they have fuel so we don't have to do an if test here.
			--levelBatteryFuel;

			// Deactivate the torch if the player runs out of fuel.
			if (levelBatteryFuel == 0)
			{
				activateLightByType(playerLight, FALSE);
			}

			// Reset step count to 0.
			batteryPlayerSteps = 0;
		}
	}
}