#include "Inventory.h"

u8 inventory[INVENTORY_LINE_COUNT];
u8 inventoryItemCount;
u8 inventoryLeftItem;
u8 inventoryRightItem;
u8 inventoryToDropItem;
u8 persistentInventory2;
u8 persistentInventory3;
u8 persistentInventoryLeftItem;
u8 persistentInventoryRightItem;

void initInventory()
{
	inventoryItemCount = 0;
	INVENTORY_LINE_1 = 0;
	INVENTORY_LINE_2 = 0;
	INVENTORY_LINE_3 = 0;
	inventoryLeftItem = 0;
	inventoryRightItem = 0;
	inventoryToDropItem = 0;
	PERSISTENT_INVENTORY_LINE_2 = 0;
	PERSISTENT_INVENTORY_LINE_3 = 0;
	persistentInventoryLeftItem = 0;
	persistentInventoryRightItem = 0;
}

void initLevelInventory()
{
	inventoryItemCount = 0;
	INVENTORY_LINE_1 = 0;
	INVENTORY_LINE_2 = 0;
	INVENTORY_LINE_3 = 0;
	inventoryToDropItem = 0;
}

void saveInventory()
{
	// To retain line 2: INVENTORY_ITEM_SCUBA, INVENTORY_ITEM_OIL_LAMP, INVENTORY_ITEM_ELECTRIC_TORCH
	// 1 + 8 + 32 = 41
	PERSISTENT_INVENTORY_LINE_2 = INVENTORY_LINE_2 & 41;

	// To retain line 3: INVENTORY_ITEM_ONE_HANDED_SWORD, INVENTORY_ITEM_TWO_HANDED_SWORD, INVENTORY_ITEM_BOW
	// 1 + 2 + 4 = 7
	PERSISTENT_INVENTORY_LINE_3 = INVENTORY_LINE_3 & 7;
}

void restoreInventory()
{
	INVENTORY_LINE_2 = PERSISTENT_INVENTORY_LINE_2;
	INVENTORY_LINE_3 = PERSISTENT_INVENTORY_LINE_3;

	// Calculate the number of items remaining.
	inventoryItemCount = bitCount(INVENTORY_LINE_2) + bitCount(INVENTORY_LINE_3);
}

void restoreEquipedItems()
{
	// Restore the inventory items carried from the persistent values following level end.
	inventoryLeftItem = persistentInventoryLeftItem;
	inventoryRightItem = persistentInventoryRightItem;
}

void saveEquipedItems()
{
	 persistentInventoryLeftItem = inventoryLeftItem;
	 persistentInventoryRightItem = inventoryRightItem;
}

void addItemToInventory(u8 cell)
{
	// Don't add the item to the inventory if it is light fuel.
	if (cell < 3 || cell == ITEM_LAMP_OIL || cell == ITEM_ELECTRIC_TORCH_BATTERY || cell == ITEM_ARROW)
	{
		return;
	}

	if (cell < 11)
	{
		INVENTORY_LINE_1 |= (1 << (cell - 3));
	}
	else if (cell < 19)
	{
		INVENTORY_LINE_2 |= (1 << (cell - 11));
	}
	else if (cell < 24)
	{
		INVENTORY_LINE_3 |= (1 << (cell - 19));
	}

	++inventoryItemCount;
}

void removeItemFromInventory(u8 cell)
{
	if (cell < 11)
	{
		INVENTORY_LINE_1 &= ~(1 << (cell - 3));
	}
	else if (cell < 19)
	{
		INVENTORY_LINE_2 &= ~(1 << (cell - 11));
	}
	else if (cell < 24)
	{
		INVENTORY_LINE_3 &= ~(1 << (cell - 19));
	}

	--inventoryItemCount;
}

void equipItem(u8 line, u8 flag, u8 equipPosition)
{
	// Calculate the item type number.
	u8 item = inventoryFlagToItemType(line, flag);

	if (item == ITEM_BOW || item == ITEM_TWO_HANDED_SWORD || item == ITEM_WOOD_BLOCK || item == ITEM_SCUBA)
	{
		// These items are equipped in both hands.
		inventoryLeftItem = item;
		inventoryRightItem = item;
	}
	else if (item == ITEM_OIL_LAMP || item == ITEM_ELECTRIC_TORCH || item == ITEM_ONE_HANDED_SWORD)
	{
		// These items are equipped in only one hand.
		if (equipPosition == EQUIP_ITEM_LH)
		{
			inventoryLeftItem = item;

			// A bow must be unequipped if any other item is equipped.
			if (inventoryRightItem == ITEM_BOW || inventoryRightItem == ITEM_TWO_HANDED_SWORD || inventoryRightItem == ITEM_SCUBA)
			{
				inventoryRightItem = 0;
			}
		}
		else
		{
			inventoryRightItem = item;

			// A bow must be unequipped if any other item is equipped.
			if (inventoryLeftItem == ITEM_BOW || inventoryLeftItem == ITEM_TWO_HANDED_SWORD || inventoryLeftItem == ITEM_SCUBA)
			{
				inventoryLeftItem = 0;
			}
		}
	}

	// Reset to drop item to 0 if item is equipped.
	if (item == inventoryToDropItem)
	{
		inventoryToDropItem = 0;
	}	
}

void dropItem(u8 item)
{
	// Only certain light items, the wood block and the carry mine can be dropped.
	if (item == ITEM_OIL_LAMP || item == ITEM_ELECTRIC_TORCH || item == ITEM_WOOD_BLOCK || item == ITEM_CARRY_MINE)
	{
		// Unequip the item.
		unequipItem(item);

		// Update to drop variable.
		inventoryToDropItem = item;
	}	
}

void unequipItem(u8 item)
{
	// Only unequip the item if it is equipped.
	if (inventoryLeftItem == item)
	{
		inventoryLeftItem = 0;
	}

	if (inventoryRightItem == item)
	{
		inventoryRightItem = 0;
	}

	if (inventoryToDropItem == item)
	{
		inventoryToDropItem = 0;
	}
}

u8 inventoryFlagToItemType(u8 line, u8 flag)
{
	// Calculate the item type number.
	u8 item = 0;

	// Work out the number of the bit from the flag.
	while (flag >>= 1)
	{
		++item;
	}

	// The items that can be in the inventories start at 3, 8 items per line.
	item += 3 + line * 8;

	return item;
}

u8 inventoryContainsItem(u8 cell)
{
	if (cell < 11)
	{
		return INVENTORY_LINE_1 & (1 << (cell - 3));
	}
	else if (cell < 19)
	{
		return INVENTORY_LINE_2 & (1 << (cell - 11));
	}
	else if (cell < 24)
	{
		return INVENTORY_LINE_3 & (1 << (cell - 19));
	}
	else
	{
		return 0;
	}
}