#ifndef INVENTORY_H
#define INVENTORY_H

#include <types.h>
#include "Item.h"
#include "Utility.h"

#define INVENTORY_ITEM_KEY_RED 1
#define INVENTORY_ITEM_KEY_GREEN 2
#define INVENTORY_ITEM_KEY_YELLOW 4
#define INVENTORY_ITEM_HAMMER 8
#define INVENTORY_ITEM_GLOVE 16
#define INVENTORY_ITEM_KEY_CARD_RED 32
#define INVENTORY_ITEM_KEY_CARD_GREEN 64
#define INVENTORY_ITEM_KEY_CARD_YELLOW 128
#define INVENTORY_ITEM_SCUBA 1
#define INVENTORY_ITEM_POTION 2
#define INVENTORY_ITEM_NAIL_BOX 4
#define INVENTORY_ITEM_OIL_LAMP 8
#define INVENTORY_ITEM_LAMP_OIL 16
#define INVENTORY_ITEM_ELECTRIC_TORCH 32
#define INVENTORY_ITEM_ELECTRIC_TORCH_BATTERY 64
#define INVENTORY_ITEM_WOOD_BLOCK 128
#define INVENTORY_ITEM_ONE_HANDED_SWORD 1
#define INVENTORY_ITEM_TWO_HANDED_SWORD 2
#define INVENTORY_ITEM_BOW 4
#define INVENTORY_ITEM_ARROW 8
#define INVENTORY_ITEM_CARRY_MINE 16

#define INVENTORY_LINE_COUNT 3
#define INVENTORY_LINE_1 inventory[0]
#define INVENTORY_LINE_2 inventory[1]
#define INVENTORY_LINE_3 inventory[2]
#define PERSISTENT_INVENTORY_LINE_2 persistentInventory2
#define PERSISTENT_INVENTORY_LINE_3 persistentInventory3

#define EQUIP_ITEM_LH 1
#define EQUIP_ITEM_RH 2

extern u8 inventoryLeftItem;
extern u8 inventoryRightItem;

// Item type for the item being dropped from the inventory to place in the map.
extern u8 inventoryToDropItem;

// Using flags for inventory on the assumption it is more memory efficient.
extern u8 inventory[INVENTORY_LINE_COUNT];

// The number of items currently held in the inventory.
extern u8 inventoryItemCount;

// Inventory lines for items that carry over across levels.
extern u8 persistentInventory2;
extern u8 persistentInventory3;

// Persistant inventory items currently held.
extern u8 persistentInventoryLeftItem;
extern u8 persistentInventoryRightItem;

// Initialise inventory values for the start of a game session.
void initInventory();

// Initialise in ventory values for the start of a new level.
void initLevelInventory();

// Save inventory items that need to be persistent between levels.
void saveInventory();

// Restore inventory item values for items that need to be persistent between levels.
void restoreInventory();

// Restore persistent items from the equiped items.
void restoreEquipedItems();

// Saves currently equiped items to the persistent variables to be restored later as needed.
void saveEquipedItems();

// Adds a specificed item to the inventory from a given cell graphic value without flags.
void addItemToInventory(u8 cell);

// Removes a specificed item from the inventory for a given cell graphic value without flags.
void removeItemFromInventory(u8 cell);

// Equip an item for the player to carry.
void equipItem(u8 line, u8 flag, u8 equipPosition);

// Removes the specified item from the inventory so that it can be placed on screen.
void dropItem(u8 item);

// Unequip the specified item if it is currently equipped.
void unequipItem(u8 item);

// Converts and inventory flag and line into an item type.
u8 inventoryFlagToItemType(u8 line, u8 flag);

// Checks if the inventory contains the specified item type.
u8 inventoryContainsItem(u8 cell);

#endif