#ifndef LIGHTING_H
#define LIGHTING_H

#include <cpctelera.h>
#include "Cell.h"
#include "Difficulty.h"
#include "GameTypes.h"
#include "Inventory.h"
#include "Item.h"
#include "Object.h"
#include "Player.h"
#include "Tile.h"

// 3 player light sources, 3 static map light sources.
#define MAX_LIGHT_SOURCE_COUNT 6

#define STATIC_LIGHT_SOURCE_TYPE 29

// Determines when the lighting should be run or not.
extern u8 lightingChange;

// Actual light source count.
extern u8 lightSourceCount;

// The static light sources on the map.
extern u8* mapStaticLightSources;

// The number of static light sources on the map.
extern u8 mapStaticLightSourceCount;

typedef struct LightSource LightSource;

struct LightSource
{
    Position position;
    Position previousPosition;
    u8 direction;
    u8 previousDirection;
    u8 type;
    u8 active;
};

extern LightSource lightSources[MAX_LIGHT_SOURCE_COUNT];

// Initialises all the static light sources loaded from the map. 
void initLighting();

// Sets a light to active or inactive based on the type.
void activateLightByType(u8 type, u8 active);

// Sets a light to active or inactive based on the position.
void activateLightByPosition(u8 x, u8 y, u8 active);

// Updates a light following a player move.
void updateLightPosition(u8 type, u8 x, u8 y, u8 direction);

// Registers a new light source into the system.
void registerLightSource(u8 type, u8 x, u8 y, u8 direction);

// Clears all the light sources removing them.
void clearLightSources();

// Runs through all the lighting logic in sequence.
void runLighting();

// Indicates if the specified light type is already registered.
u8 isLightRegistered(u8 type);

void displayLight(u8 playerX, u8 playerY, u8 activeLightType, u8 lightOn, u8 direction);

// Creates the light going left starting at origin x,y and going outward.
// This puts the form for example in 7,3,5,1.
void lightLeftAlternate(u8 x, u8 y, u8 cellsOutY, u8 lightOn);

// Creates the light going right starting at an origin x,y and going outward.
// This puts the form for example in 7,3,5,1.
void lightRightAlternate(u8 x, u8 y, u8 cellsOutY, u8 lightOn);

// Creates the light going north from a position x,y in the form of a 3x4 rectangle depending on cellsOutY. (upward)
void lightNorth(u8 x, u8 y, u8 cellsOutY, u8 lightOn);

// Creates the light going south from a position x,y in the form of a 3x4 rectangle depending on cellsOutY. (downward)
void lightSouth(u8 x, u8 y, u8 cellsOutY, u8 lightOn);

// Creates the torch light beam going east from a position x,y in the form of a 5x3 depnding on width and length. (right and out)
void lightBeamEast(u8 x, u8 y, u8 beamLength, u8 beamWidth, u8 lightOn);

// Creates the torch light beam going west from a position x,y in the form of a 5x3 depnding on width and length. (left and out)
void lightBeamWest(u8 x, u8 y, u8 beamLength, u8 beamWidth, u8 lightOn);

// Returns the item type for the light that the player has currently got equipped.
u8 lightEquipped();

// Retrieves the position x,y of the nearest light source.
u8 nearestLight(u8* x, u8* y);

#endif