#include "Menu.h"

#define MENU_ITEM_COUNT_ 5

u8 currentMenuItem_;

const u8* COLOUR_MODE_MENU_TEXT_;
const u8 GREEN_MODE_TEXT_[12] = "green mode \0";
const u8 COLOUR_MODE_TEXT_[12] = "colour mode\0";
const u8* difficultyMenuText_;
const u8 EASY_DIFFICULTY_TEXT_[16] = "easy difficulty";
const u8 NORMAL_DIFFICULTY_TEXT_[18] = "normal difficulty";
const u8 HARD_DIFFICULTY_TEXT_[16] = "hard difficulty";

#define SCORE_COUNT_ 5
#define MAX_NAME_LENGTH_ 10

u16 highScores_[SCORE_COUNT_];
u8 highScoreNames_[SCORE_COUNT_][11];

#define START_GAME_ 0
#define REBIND_CONTROLS_ 1
#define DIFFICULTY_ 2
#define HIGH_SCORES_ 3
#define COLOUR_MODE_ 4

#define GAME_VERSION_ "1.0"

void runMenu()
{
    //setup the default keys for the player
	setupDefaultKeys();

    //set the default high score entries
    initScores();

    //default graphics mode is colour mode
    COLOUR_MODE_MENU_TEXT_ = COLOUR_MODE_TEXT_;

	// Default difficulty is normal difficulty.
	chosenDifficulty = NORMAL_DIFFICULTY;
	difficultyMenuText_ = NORMAL_DIFFICULTY_TEXT_;

    //draw the menu initially on entry to the main menu
    drawMenu();

    //The first menu item to be selected
	currentMenuItem_ = 0;

    //menu loop
    while(1)
    {
        if(cpct_isKeyPressed(Key_Return) || cpct_isKeyPressed(gameControls.use))
        {
            switch(currentMenuItem_)
            {
                case START_GAME_:
                {
					// Set difficulty to the chosen difficulty.
					setupDifficulty();

                    //runs the full game session
					runGameSession();

					// Make sure the interrupts won't change the palette.
					waterLevel = FALSE;

                    //this causes victory screen to be shown
                    //currentLevel = GAME_COMPLETE;

					// Set the palette to the standard palette.
					choosePalette(MENU_PALETTE);

                    //display the end screen
					displayEndScreen();

                    //update and show high scores
                    updateHighScores();
                }
                break;

                case REBIND_CONTROLS_:
                    rebindControls();
                break;

				case DIFFICULTY_:
					chooseDifficulty();
				break;

                case HIGH_SCORES_:
					displayHighScores();
                break;

                case COLOUR_MODE_:
                    chooseGraphicMode();
                break;
            }

            //clear the screen after the previous option ends
            cpct_clearScreen(0);

            //redraw the menu following the previous option ends
            drawMenu();

            //wait for enter not to be pressed any more
            while(cpct_isKeyPressed(Key_Return) || cpct_isKeyPressed(gameControls.use))
            {
            }
        }
        else if(cpct_isKeyPressed(gameControls.up))
        {
            //0 is the minimum
            if(currentMenuItem_)
            {
                --currentMenuItem_;

				//update indicator position
				drawIndicator();
            }

			//wait for up not to be pressed any more
            while(cpct_isKeyPressed(gameControls.up))
            {
            }
        }
        else if(cpct_isKeyPressed(gameControls.down))
        {
            //item count cant reach max menu item count
            if(currentMenuItem_ + 1 < MENU_ITEM_COUNT_)
            {
                ++currentMenuItem_;

				//update indicator position
				drawIndicator();
            }
            
			//wait for down not to be pressed any more
            while(cpct_isKeyPressed(gameControls.down))
            {
            }
        }
    }
}

void drawMenu()
{
	// Set the palette to the standard palette.
	choosePalette(MENU_PALETTE);

	// Set the border colour so that we can see the map edge more easily.
	cpct_setBorder(6);

    // Game title.
    drawString(20, 5, "fitzroy in the dark");

    //actual menu options
    drawString(25, 45, "start game");
    drawString(25, 60, "rebind controls");
    drawString(25, 75, difficultyMenuText_);
    drawString(25, 90, "high scores");    
    drawString(25, 105, COLOUR_MODE_MENU_TEXT_);

    //the dot indicating which menu option is selected
	drawIndicator();

	//draw the game version number on the bottom right corner of the screen
	drawString(70, 180, GAME_VERSION_);
}

void drawIndicator()
{
	u8 x = 20;
	u8 y = 40;

	//draw over where the indicator can go
	u8* memory = cpct_getScreenPtr(CPCT_VMEM_START, x, y);
	
	cpct_drawSolidBox(memory, cpct_px2byteM0(0, 0), TILE_WIDTH_BYTES, 15 * MENU_ITEM_COUNT_);

	//draw the current position of the indicator
	y = 42 + currentMenuItem_ * 15;
	memory = cpct_getScreenPtr(CPCT_VMEM_START, x, y);
	
	cpct_drawSprite(G_item_torch, memory, TILE_WIDTH_BYTES, TILE_HEIGHT);
}

void rebindControls()
{
    cpct_clearScreen(0);

    //title
    drawString(15,8, "enter controls");

    //draw the text, make sure no key is currently down then set the key for the next control (for each getKey call)
    drawString(25,30, "up");
    gameControls.up = getKey();

    drawString(25,45, "down");
    gameControls.down = getKey();

    drawString(25,60, "left");
    gameControls.left = getKey();

    drawString(25,75, "right");
    gameControls.right = getKey();

    drawString(25,90, "use");
    gameControls.use = getKey();

	drawString(25,105, "attack");
    gameControls.attack = getKey();

	drawString(25,120, "inventory");
	gameControls.inventory = getKey();

	drawString(25,135, "exit");
	gameControls.exit = getKey();
}

cpct_keyID getKey()
{
	// From Ronaldo and Arnaud on cpcwiki forum
	u8* keyStatus = cpct_keyboardStatusBuffer;
	u8 i;

	// wait for any previous key presses to clear first
    while (cpct_isAnyKeyPressed());

    //then wait for the desired key press
    while (!cpct_isAnyKeyPressed());

	for (i = 0; i < 10; ++i)
	{
		cpct_keyID keypressed = *keyStatus++ ^ 0xFF;
		if (keypressed)
        {
            return (keypressed << 8) + i;
        }
	}

	return 0;
}

void chooseGraphicMode()
{
	// Update menu item text to reflect the chosen graphic mode.
	if (greenMode)
	{
		greenMode = FALSE;
		COLOUR_MODE_MENU_TEXT_ = COLOUR_MODE_TEXT_;
	}
	else
	{
		greenMode = TRUE;
		COLOUR_MODE_MENU_TEXT_ = GREEN_MODE_TEXT_;
	}

	choosePalette(STANDARD_PALETTE);
}

void initScores()
{
    u8 i = 0;

    for(i = 0; i<SCORE_COUNT_; ++i)
    {
        highScores_[i] = 10;
        cpct_memcpy(highScoreNames_[i], "cpc\0", 4);
    }
}

void displayHighScores()
{
    u8 i = 0;	

    cpct_clearScreen(0);

    //title
    drawString(25, 5, "high scores");

    //entries
    for(i = 0; i<SCORE_COUNT_; ++i)
    {
		u8 scoreStart = 25;
		u8 output[6];
		u8 y = 45 + i * 15;

		// Draw the name of the person.
        drawString(40, y, highScoreNames_[i]);
		
		// Convert the score to a string.
		convertIntToChar(highScores_[i], output);
		
		// Draw all zeros.
		drawString(scoreStart, y, "000000");

		// Determines where the score is drawn so there are 0s in the right place.
		if (highScores_[i] < 100)
		{
			scoreStart += 6;
		}
		else if (highScores_[i] < 1000)
		{
			scoreStart += 4;
		}
		else if (highScores_[i] < 10000)
		{
			scoreStart += 2;
		}

		drawString(scoreStart, y, output);
    }

	getKey();
}

void updateHighScores()
{
	//check that the player got a higher score than the bottom score
	if (gameScore > highScores_[4])
	{
		//enter name
		u8 letterCount = 0;
		u8 scoreName[11];
        u8 highScoreText[6];
		u8 i = 0;

		//default to anything
		cpct_keyID chosenKey = Key_0;
				
		//set scoreName to the null terminator
		cpct_memset(scoreName, '\0', 11);
		
		cpct_clearScreen(0);
		drawString(15, 5, "new high score");
        convertIntToChar(gameScore, highScoreText);
        drawString(45, 5, highScoreText);
        drawString(20, 25, "enter your name");

		do
		{
			//get the next character from the user
			chosenKey = getKey();

			//del to move back to the previous key
			if (chosenKey == Key_Del || chosenKey == 16386) //16386 is wincpctelera specific
			{
				//go back to the previous letter
				if (letterCount)
				{
					--letterCount;

					scoreName[letterCount] = ' ';

					//draw letter
					drawString(25 + letterCount * 2, 50, &scoreName[letterCount]);
				}
			}
			//make sure the key is in the correct range and the word is not too long
			else if (letterCount < MAX_NAME_LENGTH_)
			{
				u8 chosenCharacter = 0;

				switch (chosenKey)
				{
					case Key_A:
						chosenCharacter = 'a';
					break;

					case Key_B:
						chosenCharacter = 'b';
					break;

					case Key_C:
						chosenCharacter = 'c';
					break;

					case Key_D:
						chosenCharacter = 'd';
					break;

					case Key_E:
						chosenCharacter = 'e';
					break;

					case Key_F:
						chosenCharacter = 'f';
					break;

					case Key_G:
						chosenCharacter = 'g';
					break;

					case Key_H:
						chosenCharacter = 'h';
					break;

					case Key_I:
						chosenCharacter = 'i';
					break;

					case Key_J:
						chosenCharacter = 'j';
					break;

					case Key_K:
						chosenCharacter = 'k';
					break;

					case Key_L:
						chosenCharacter = 'l';
					break;

					case Key_M:
						chosenCharacter = 'm';
					break;

					case Key_N:
						chosenCharacter = 'n';
					break;

					case Key_O:
						chosenCharacter = 'o';
					break;

					case Key_P:
						chosenCharacter = 'p';
					break;

					case Key_Q:
						chosenCharacter = 'q';
					break;

					case Key_R:
						chosenCharacter = 'r';
					break;

					case Key_S:
						chosenCharacter = 's';
					break;

					case Key_T:
						chosenCharacter = 't';
					break;

					case Key_U:
						chosenCharacter = 'u';
					break;

					case Key_V:
						chosenCharacter = 'v';
					break;

					case Key_W:
						chosenCharacter = 'w';
					break;

					case Key_X:
						chosenCharacter = 'x';
					break;

					case Key_Y:
						chosenCharacter = 'y';
					break;

					case Key_Z:
						chosenCharacter = 'z';
					break;

						//space does not work on wincpctelera
					case Key_Space:
						chosenCharacter = ' ';
					break;
				}

				if (chosenCharacter)
				{
					scoreName[letterCount] = chosenCharacter;

					//draw letter
					drawString(25 + letterCount * 2, 50, &scoreName[letterCount]);

					++letterCount;
				}
			}
		}
		while (chosenKey != Key_Return && chosenKey != Key_Enter);

		//update score values
		highScores_[4] = gameScore;

		for (i = 4; i > 0; --i)
		{
			if (highScores_[i] > highScores_[i - 1])
			{
				highScores_[i] = highScores_[i - 1];
				cpct_memcpy(highScoreNames_[i], highScoreNames_[i - 1], 11);

				highScores_[i - 1] = gameScore;
				cpct_memcpy(highScoreNames_[i - 1], scoreName, 11);
			}
		}
	}
}

void displayEndScreen()
{
	// wait for any previous key presses to clear first
	while (cpct_isAnyKeyPressed());

	//clear the screen
	cpct_clearScreen(0);

	if (currentLevel == GAME_COMPLETE)
	{		
		u8* memory;
		u8 count = 255;
		u8 colour = 0;
		u8 x = 0;
		u8 y = 0;

		//patterned background
		while (count)
		{
			colour = 1 + cpct_rand() % 15;
			y = 14 * (cpct_rand() % 14);
			x = 4 * (cpct_rand() % 17);

			memory = cpct_getScreenPtr(CPCT_VMEM_START, x, y);
			cpct_drawSolidBox(memory, cpct_px2byteM0(colour, colour), 10, 12);

			--count;
		}

		//victorious label
		memory = cpct_getScreenPtr(CPCT_VMEM_START, 25, 60);
		cpct_drawSolidBox(memory, cpct_px2byteM0(0, 0), 30, 20);
		drawString(30, 65, "victorious");

		//1000 (after the extra 0 is displayed) points for winning the game
		gameScore += 100;
	}
	else
	{
		drawString(30, 20, "game over");
	}

	//then wait for the desired key press
	while (!cpct_isAnyKeyPressed());
}

void chooseDifficulty()
{
	// Switch between difficulties and upate the menu text.
	if (chosenDifficulty == EASY_DIFFICULTY)
	{
		chosenDifficulty = NORMAL_DIFFICULTY;
		difficultyMenuText_ = NORMAL_DIFFICULTY_TEXT_;
	}
	else if (chosenDifficulty == NORMAL_DIFFICULTY)
	{
		chosenDifficulty = HARD_DIFFICULTY;
		difficultyMenuText_ = HARD_DIFFICULTY_TEXT_;
	}
	else
	{
		chosenDifficulty = EASY_DIFFICULTY;
		difficultyMenuText_ = EASY_DIFFICULTY_TEXT_;
	}
}
