#include "Player.h"
#include <stdio.h>

Player player;
Position moveIndicator_;

void initPlayer(u8 x, u8 y)
{
	//this will do as a start position for now
    player.object.position.x = x;
	player.object.position.y = y;

    //initially set next position to x,y as well
    player.object.nextPosition.x = x;
    player.object.nextPosition.y = y;

	// Set indicator to x,y (player position).
	moveIndicator_.x = x;
	moveIndicator_.y = y;
	
	//direction of 0 ie going nowhere
	player.object.direction = 0;
	player.cursorDirection = 0;

	// Default sprite direction is right.
	player.spriteDirection = DIRECTION_RIGHT;

	player.moving = FALSE;

    //set starting health for the player
    player.health = startingHealth;

	// Meat count set to 0.
	player.meat = 0;

	// Player is not blocked by default.
	player.blocked = FALSE;

	// No active projectile right now.
	activeProjectile = FALSE;
}

void drawPlayer(u8 xScroll, u8 yScroll)
{
	//draw the player test sprite
    u8* memory = cpct_getScreenPtr(CPCT_VMEM_START, (player.object.position.x - xScroll) * TILE_WIDTH_BYTES, (player.object.position.y - yScroll) * TILE_HEIGHT);

	if (player.spriteDirection == DIRECTION_RIGHT)
	{
		if (player.blocked)
		{
			cpct_drawSprite(G_player_sprite_right_blocked, memory, TILE_WIDTH_BYTES, TILE_HEIGHT);
		}
		else
		{
			cpct_drawSprite(G_player_sprite_right, memory, TILE_WIDTH_BYTES, TILE_HEIGHT);
		}
	}
	else
	{
		if (player.blocked)
		{
			cpct_drawSprite(G_player_sprite_left_blocked, memory, TILE_WIDTH_BYTES, TILE_HEIGHT);
		}
		else
		{
			cpct_drawSprite(G_player_sprite_left, memory, TILE_WIDTH_BYTES, TILE_HEIGHT);
		}		
	}
    
}

void movePlayer()
{
	// Sets the direction based on the user input.
	if (gameControls.upIsDown)
	{
		player.object.direction = DIRECTION_UP;
		player.cursorDirection = DIRECTION_UP;

		// Save the current cursor direction for the next projectile.
		projectileNextDirection = player.cursorDirection;
	}
	else if (gameControls.downIsDown)
	{
		player.object.direction = DIRECTION_DOWN;
		player.cursorDirection = DIRECTION_DOWN;

		// Save the current cursor direction for the next projectile.
		projectileNextDirection = player.cursorDirection;
	}
	else if (gameControls.leftIsDown)
	{
		player.object.direction = DIRECTION_LEFT;
		player.cursorDirection = DIRECTION_LEFT;

		// Save the current cursor direction for the next projectile.
		projectileNextDirection = player.cursorDirection;
	}
	else if (gameControls.rightIsDown)
	{
		player.object.direction = DIRECTION_RIGHT;
		player.cursorDirection = DIRECTION_RIGHT;

		// Save the current cursor direction for the next projectile.
		projectileNextDirection = player.cursorDirection;
	}
	else if(gameControls.useIsDown)
	{
		// Only set player to be moving if they have a new position.
		if(player.object.nextPosition.x != player.object.position.x || 
			player.object.nextPosition.y != player.object.position.y)
		{
			// Activates the player movement after a move has been chosen.
			player.moving = TRUE;	
		}	
	}
	else if (projectileNextDirection && gameControls.attackIsDown)
	{		
		// Launch a projectile in the chosen direction.
		if (!activeProjectile)
		{
			// Check the player has a bow equipped and has arrows.
			if (inventoryLeftItem == ITEM_BOW && levelArrowCount)
			{
				// Activate the projectile which will enable the projectile logic to run.
				activeProjectile = TRUE;
				projectile.direction = projectileNextDirection;

				// Starting the projectile at the player cursor position feels better.
				projectile.position.x = player.object.nextPosition.x;
				projectile.position.y = player.object.nextPosition.y;

				// Subtact an arrow from the count as it has been used.
				--levelArrowCount;
			}			
		}
	}
}

void logicPlayer(u8* xScroll, u8* yScroll)
{	
	// Only process the cell and perform the move if the player has chosen to move and it is their turn.
	if(gameTurnState == PLAYER_TURN && player.moving)
	{		
		u8 futureCell = processCell(player.object.nextPosition.x, player.object.nextPosition.y, *xScroll, *yScroll);

		// Reset player blocked flag.
		player.blocked = FALSE;

		// Update the player direction if it is left or right.
		if (player.object.direction == DIRECTION_LEFT || player.object.direction == DIRECTION_RIGHT)
		{
			player.spriteDirection = player.object.direction;
		}

		//let the player move over the top of an empty cell or an item
		if (!futureCell || futureCell & CELL_ITEM_FLAG)
		{
			//redraw the cell at the players previous position
			u8 previousCell = MAPA(player.object.position.x, player.object.position.y);

			drawCell(player.object.position.x - *xScroll, player.object.position.y - *yScroll, previousCell);
						
			scrollScreen(xScroll, yScroll);

			// Update player position.
			player.object.position.x = player.object.nextPosition.x;
			player.object.position.y = player.object.nextPosition.y;

			// Updates the position of the equipped light based on player position.
			updateLightPosition(lightEquipped(), player.object.position.x, player.object.position.y, player.object.direction);

			// The player has moved finishing their turn.
			gameTurnState = PROJECTILE_TURN;
		}
		else if(futureCell & CELL_INTERACTABLE_FLAG)
		{
			//redraw the cell at the players previous position		
			u8 previousCell = MAPA(player.object.position.x, player.object.position.y);

			drawCell(player.object.position.x - *xScroll, player.object.position.y - *yScroll, previousCell);

			if ((futureCell & CELL_GFX_RANGE) == INTERACTABLE_TELEPORTER_ON)
			{
				// Scrolls the screen as much as required for the teleportation.
				teleporterScroll(xScroll, yScroll);
			}
			else
			{
				// Scrolls the screen up to one cell in both directions.
				scrollScreen(xScroll, yScroll);
			}
			
			// Update player position.
			player.object.position.x = player.object.nextPosition.x;
			player.object.position.y = player.object.nextPosition.y;

			// Updates the position of the equipped light based on player position.
			updateLightPosition(lightEquipped(), player.object.position.x, player.object.position.y, player.object.direction);

			// The player has moved finishing their turn.
			gameTurnState = PROJECTILE_TURN;
		}
		else if (futureCell == INTERACTABLE_ENEMY_INDICATOR)
		{
			// Attack enemy here.
			attackEnemy(player.object.nextPosition.x, player.object.nextPosition.y, MELEE_ATTACK, *xScroll, *yScroll);

			// Make sure use\move is not down after attacking to prevent attacking multiple times.
			while (gameControls.useIsDown)
			{
				readGameControls();
			}

			// After the player has attacked the enemy it is the enemies turn to move.
			gameTurnState = PROJECTILE_TURN;
		}
		else if (futureCell == INTERACTABLE_BOSS_INDICATOR && ((futureCell & CELL_GFX_RANGE) > 3))
		{
			// Attack boss here but only if it is a dragon or gesture boss.
			attackBoss(player.object.nextPosition.x, player.object.nextPosition.y, MELEE_ATTACK, *xScroll, *yScroll);

			// Make sure use\move is not down after attacking to prevent attacking multiple times.
			while (gameControls.useIsDown)
			{
				readGameControls();
			}

			// After the player has attacked the boss it is the projectile turn.
			gameTurnState = PROJECTILE_TURN;
		}
		else
		{
			//reset player next position
			player.object.nextPosition.x = player.object.position.x;
			player.object.nextPosition.y = player.object.position.y;

			// Player was blocked by something so set blocked flag to true.
			player.blocked = TRUE;

			// The player hasn't moved so it is still their turn.
			gameTurnState = PLAYER_TURN;			
		}

		// Player has finished their turn.
		if (gameTurnState != PLAYER_TURN)
		{
			u8 cell = MAPA(player.object.position.x, player.object.position.y);

			// Dark move count is increased every time the player moves in the dark.
			if (cell & CELL_LIGHT_FLAG)
			{
				darkMoveCount = maxDarkMoves;
			}
			else
			{
				--darkMoveCount;

				// The player dies once the count reaches zero.
				if (darkMoveCount == 0)
				{
					player.health = 0;
				}
			}

			// Update the fuel based on the number of steps fitzroy has moved.
			updateFuelState();

			// Dark move count has changed so redraw hud.
			hudRedraw = TRUE;
		}

		//rest movement
		player.moving = FALSE;		
	}

	// Player on 0 health so they will be kicked out of the level.
	if (!player.health)
	{
		// The level has been ended in death.
		levelEnded = LEVEL_END_DEATH;
		
		// Restore previously equiped values as the player has ended their current sesion in death.
		restoreEquipedItems();

		// Clear all light sources so new ones can be initialised on level start.
		clearLightSources();

		// Set the current level value to 255 to indicate the player has died and is not moving to the next level.
		currentLevel = 255;
	}
}

u8 processCell(u8 x, u8 y, u8 xScroll, u8 yScroll)
{
	u8 cell = MAPA(x, y);

	if(cell & CELL_ITEM_FLAG)
    {
		if ((cell & CELL_GFX_RANGE) == ITEM_WOOD_BLOCK)
		{
			// The wood block cannot be picked up while any other item is equipped.
			if (inventoryLeftItem || inventoryRightItem)
			{
				// Returning a 1 is equivalent to a tile (removing item flag) so that the player cannot pickup the item until they unequipped their items.
				return 1;
			}

			// Set the wood block to be equipped in both hands.
			inventoryLeftItem = ITEM_WOOD_BLOCK;
			inventoryRightItem = ITEM_WOOD_BLOCK;
		}
		else if (inventoryContainsItem(cell & CELL_GFX_RANGE))
		{
			// Returning a 1 is equivalent to a tile (removing item flag) so that the player cannot pickup the item matching the one in the inventory.
			return 1;
		}

        // Found an item so add it to the level score.
		levelScore += itemScore(cell & CELL_GFX_RANGE);

		// Consume the item if there is any player stats to update.
		consumeItem(cell & CELL_GFX_RANGE);

		//get the hud to redraw with the updated score
		hudRedraw = TRUE;

		// Reset the cell to blank (removes the item) but only visible if it is currently visible.
		if (cell & CELL_LIGHT_FLAG)
		{
			mapCells[XY_TO_POSITION(x, y)] = postMoveCell | CELL_LIGHT_FLAG;
		}
		else
		{
			mapCells[XY_TO_POSITION(x, y)] = postMoveCell;
		}

		//redraw a blank cell for redraw
		drawBlankCell(x - xScroll, y - yScroll);

		// Add the item to the inventory for all supported persistant items.
		addItemToInventory(cell & CELL_GFX_RANGE);

		// Deactivates the itme if it is a registered light item.
		activateLightByType(cell & CELL_GFX_RANGE, FALSE);		
    }
    else if(cell & CELL_INTERACTABLE_FLAG)
    {
		// Strip off the flag as we don't need or want it.
		u8 interactableGraphic = cell & CELL_GFX_RANGE;

		// Test to see if the interactable has met its conditions for activation.
		if(isInteractableReady(interactableGraphic))
		{
			//get the hud to redraw with the updated score
			hudRedraw = TRUE;

			// Only doors get reset to blank (we can use <= because doors are 1,2 and 3).
			if(interactableGraphic <= INTERACTABLE_DOOR_YELLOW)
			{
				// Reset the cell to blank (removes the interactable) but only visible if it is currently visible.
				if (cell & CELL_LIGHT_FLAG)
				{
					mapCells[XY_TO_POSITION(x, y)] = postMoveCell | CELL_LIGHT_FLAG;
				}
				else
				{
					mapCells[XY_TO_POSITION(x, y)] = postMoveCell;
				}

				//redraw a blank cell for redraw
				drawBlankCell(x - xScroll, y - yScroll);
			}
			else if (interactableGraphic == INTERACTABLE_SWITCH_ON)
			{
				// Change interactable graphic from on switch to off switch.
				mapCells[XY_TO_POSITION(x, y)] = INTERACTABLE_SWITCH_OFF | CELL_INTERACTABLE_FLAG;

				// Apply the light flag if it is currently set on the cell.
				if (cell & CELL_LIGHT_FLAG)
				{
					mapCells[XY_TO_POSITION(x, y)] |= CELL_LIGHT_FLAG;
				}
			}
			else if (interactableGraphic == INTERACTABLE_SWITCH_OFF)
			{
				// Change interactable graphic from off switch to on switch.
				mapCells[XY_TO_POSITION(x, y)] = INTERACTABLE_SWITCH_ON | CELL_INTERACTABLE_FLAG;

				// Apply the light flag if it is currently set on the cell.
				if (cell & CELL_LIGHT_FLAG)
				{
					mapCells[XY_TO_POSITION(x, y)] |= CELL_LIGHT_FLAG;
				}
			}
			else if (interactableGraphic == INTERACTABLE_LASER_GATE)
			{
				// Player touches a laser gate killing them instantly.
				player.health = 0;
			}
			else if (interactableGraphic == INTERACTABLE_MINE)
			{
				// Player touches a mine killing them instantly.
				player.health = 0;

				// Reset the timer.
				timerCount = 0;

				// Indicate there is a death delay going on.
				deathDelay = TRUE;

				// Display the mine explosion where the mine is.
				cell = CELL_LIGHT_FLAG + TILE_MINE_EXPLOSION;

				// Draw the explosion at the mine cell.
				drawCell(x - xScroll, y - yScroll, cell);

				while (deathDelay)
				{
					// Wait for the timer to reset so we can show the player the mine explosion.
				}
			}
			else if (interactableGraphic == INTERACTABLE_TELEPORTER_ON)
			{
				if (playerTeleportingX != 255)
				{
					// Return acitve teleporter to indicate that a teleporation has occured.
					return INTERACTABLE_TELEPORTER_ON | CELL_INTERACTABLE_FLAG;
				}
			}
			else if (interactableGraphic == INTERACTABLE_CAVE_ENTRANCE_OPEN || interactableGraphic == INTERACTABLE_CAVE_ENTRANCE_COMPLETED)
			{
				// Note currently supporting re-entering completed level.

				// Player reached the exit\entrance.
				levelEnded = LEVEL_END_COMPLETE;

				// Save inventory items that need to be persistent.
				saveInventory();

				// Save the items currently held to the persistent values.
				saveEquipedItems();

				// Updates the score following completing a level.
				updateGameSessionState();

				if (currentLevel == ENTRANCE_CAVE_LEVEL)
				{
					// Save the position before entering the entrance cave for later.
					preLevelTransitionPositionX = player.object.position.x;
					preLevelTransitionPositionY = player.object.position.y;
				}
				else if (currentLevel == BOSS_LEVEL)
				{
					currentLevel = GAME_COMPLETE;
				}
				else
				{
					// Add 500 (after the extra 0 is displayed) points for completing the level.
					gameScore += 50;

					// Set the finished level to be complete.
					// -1 becasue the entrance cave cannot be complete as such.
					caveComplete[currentLevel - 1] = TRUE;
				}

				// Update player position to be equal to cave exit\entrance.
				player.object.position.x = player.object.nextPosition.x;
				player.object.position.y = player.object.nextPosition.y;
			}
			else if (interactableGraphic == INTERACTABLE_BRIDGE_BASE_LEFT || interactableGraphic == INTERACTABLE_BRIDGE_BASE_RIGHT)
			{
				// Change interactable graphic from a bridge base to a bridge
				mapCells[XY_TO_POSITION(x, y)] = (interactableGraphic + 1) | CELL_INTERACTABLE_FLAG;

				// Apply the light flag if it is currently set on the cell.
				if (cell & CELL_LIGHT_FLAG)
				{
					mapCells[XY_TO_POSITION(x, y)] |= CELL_LIGHT_FLAG;
				}
			}
			else if (interactableGraphic == INTERACTABLE_BRIDGE_LEFT || interactableGraphic == INTERACTABLE_BRIDGE_RIGHT)
			{
				// Return an interactable that we can stand on and thus cross the bridge.
				return CELL_INTERACTABLE_FLAG;
			}
			else if (interactableGraphic == INTERACTABLE_WATER_BLOCKER)
			{
				// Reset the cell to water (removes the interactable) but only visible if it is currently visible.
				if (cell & CELL_LIGHT_FLAG)
				{
					mapCells[XY_TO_POSITION(x, y)] = TILE_WATER_VERTICAL | CELL_LIGHT_FLAG;
				}
				else
				{
					mapCells[XY_TO_POSITION(x, y)] = TILE_WATER_VERTICAL;
				}
			}
			else if (interactableGraphic == INTERACTABLE_ENEMY_INDICATOR)
			{
				// Return the enemy indicator to be processed in the logic function.
				return INTERACTABLE_ENEMY_INDICATOR;
			}
			else if (interactableGraphic == INTERACTABLE_GESTURE_PLANT)
			{
				// The plant has been appeased so you can move through.
				return 0;
			}
			else if (interactableGraphic == INTERACTABLE_CARD_READER_RED || interactableGraphic == INTERACTABLE_CARD_READER_GREEN || interactableGraphic == INTERACTABLE_CARD_READER_YELLOW)
			{
				// Once used the card reader is set to off.
				if (cell & CELL_LIGHT_FLAG)
				{
					MAPA(x, y) = INTERACTABLE_CARD_READER_OFF | CELL_INTERACTABLE_FLAG | CELL_LIGHT_FLAG;
				}
				else
				{
					MAPA(x, y) = INTERACTABLE_CARD_READER_OFF | CELL_INTERACTABLE_FLAG;
				}

				return cell & 0x3F;
			}
			else if (interactableGraphic == INTERACTABLE_DIRT_CRACK)
			{
				// Reset the cell to impassable (removes the interactable) but only visible if it is currently visible.
				if (cell & CELL_LIGHT_FLAG)
				{
					mapCells[XY_TO_POSITION(x, y)] = TILE_IMPASSABLE | CELL_LIGHT_FLAG;
				}
				else
				{
					mapCells[XY_TO_POSITION(x, y)] = TILE_IMPASSABLE;
				}

				// The player will be able to move onto the interactable tile.
				return 0;
			}
		}
		else
		{
			// Returning a 1 is equivalent to a tile (removing interactable flag) so that the player cannot move over the interactable.
			return 1;
		}
    }
	else if ((cell & 0x3F) >= CELL_LIGHT_BLOCKER_GFX_RANGE)
	{
		// Tiles outside of the CELL_LIGHT_BLOCKER_GFX_RANGE are treated as empty and can be stood on.		
		if ((cell & CELL_GFX_RANGE) == TILE_WATER_VERTICAL || (cell & CELL_GFX_RANGE) == TILE_WATER_HORIZONTAL)
		{
			// Check if player has equipped the scuba gear.
			if (inventoryLeftItem == ITEM_SCUBA)
			{
				return 0;
			}

			// Not carrying scuba gear so can't go in the water.
			return 1;
		}
		else if((cell & CELL_GFX_RANGE) == TILE_IMPASSABLE)
		{
			// The cell is impassable and cannot be stood on.
			return 1;
		}
		else
		{
			return 0;
		}		
	}

	// Return the cell where we can use the type information as we need but without the light flag.
	return cell & 0x3F;
}

void drawPlayerNextMoveIndicator(u8 xScroll, u8 yScroll)
{
	// Make sure the player has chosen a new position that is different to the current position.
	if(player.object.nextPosition.x != player.object.position.x || 
	   player.object.nextPosition.y != player.object.position.y) 
	{
		u8* memory = cpct_getScreenPtr(CPCT_VMEM_START, (moveIndicator_.x - xScroll) * TILE_WIDTH_BYTES, (moveIndicator_.y - yScroll) * TILE_HEIGHT);
		u8 cellLight = MAPA(moveIndicator_.x, moveIndicator_.y) & CELL_LIGHT_FLAG;

		if ((currentLevel == GROWING_PLANT_LEVEL || currentLevel == GESTURE_LEVEL) && !cellLight)
		{
			// On the growing plant level for dark squares we have to show the white indicator as the default is black.
			cpct_drawSpriteMasked(G_highlighted_mask, memory, TILE_WIDTH_BYTES, TILE_HEIGHT);
		}
		else
		{
			// Normally shows the standard indicator which is currently white on all levels expect plant levels.
			cpct_drawSpriteMasked(G_move_indicator, memory, TILE_WIDTH_BYTES, TILE_HEIGHT);
		}		
	}
}

void determinePlayerNextMove(u8 xScroll, u8 yScroll)
{
	// Doing this prevents the indicator moving multiple squares with a single keypress.
	if (gameControls.upIsDown || gameControls.downIsDown || gameControls.leftIsDown || gameControls.rightIsDown)
	{
		return;
	}

	if (player.cursorDirection == DIRECTION_LEFT)
	{
		// The next move can only ever be one cell away from the current position.
		if ((player.object.position.x - (player.object.nextPosition.x - 1)) < 2)
		{
			--player.object.nextPosition.x;
		}
				
		//move left (255 is effectively -1)
		if (player.object.nextPosition.x == 255)
		{
			//left boundary so reset to current position
			player.object.nextPosition.x = player.object.position.x;
		}		
	}
	else if (player.cursorDirection == DIRECTION_RIGHT)
	{
		// The next move can only ever be one cell away from the current position.
		if (((player.object.nextPosition.x + 1) - player.object.position.x) < 2)
		{
			++player.object.nextPosition.x;
		}

		//move right
		if (player.object.nextPosition.x >= MAP_TILES_X)
		{
			//right boundary so reset to current position
			player.object.nextPosition.x = player.object.position.x;
		}
	}
	else if (player.cursorDirection == DIRECTION_UP)
	{
		// The next move can only ever be one cell away from the current position.
		if ((player.object.position.y - (player.object.nextPosition.y - 1)) < 2)
		{
			--player.object.nextPosition.y;
		}		

		//move up (255 is effectively -1)
		if (player.object.nextPosition.y == 255)
		{
			//top boundary so reset to current position
			player.object.nextPosition.y = player.object.position.y;
		}
	}
	else if (player.cursorDirection == DIRECTION_DOWN)
	{
		// The next move can only ever be one cell away from the current position.
		if (((player.object.nextPosition.y + 1) - player.object.position.y) < 2)
		{
			++player.object.nextPosition.y;
		}

		//move down
		if (player.object.nextPosition.y >= MAP_TILES_Y)
		{
			//bottom boundary so reset to current position
			player.object.nextPosition.y = player.object.position.y;
		}
	}

	if (moveIndicator_.x != player.object.nextPosition.x ||
		moveIndicator_.y != player.object.nextPosition.y)
	{
		u8 cell = MAPA(moveIndicator_.x, moveIndicator_.y);

		// Draw over the old position for the move indicator.	
		drawCell(moveIndicator_.x - xScroll, moveIndicator_.y - yScroll, cell);

		// Update moveIndicator to match the new next position
		moveIndicator_.x = player.object.nextPosition.x;
		moveIndicator_.y = player.object.nextPosition.y;

		// Set enemy to be redrawn in case an enemy position has been selected.
		enemyRedraw = TRUE;

		// Set boss to be redrawn in case a boss cell has been selected.
		bossRedraw = TRUE;
	}

	// Reset cursor direction to none.	
	player.cursorDirection = 0;
}

void scrollScreen(u8* xScroll, u8* yScroll)
{
	if (player.object.nextPosition.x > player.object.position.x)
	{
		// Moving right.
		if (player.object.nextPosition.x > 9 && player.object.nextPosition.x < (MAP_TILES_X - 10))
		{
			++(*xScroll);
		}
	}
	else if (player.object.nextPosition.x < player.object.position.x)
	{
		// Moving left.
		if (player.object.nextPosition.x > 8 && player.object.nextPosition.x < (MAP_TILES_X - 11))
		{
			--(*xScroll);
		}
	}

	if (player.object.nextPosition.y > player.object.position.y)
	{
		// Moving down.
		if (player.object.nextPosition.y > 5 && player.object.nextPosition.y < (MAP_TILES_Y - 6))
		{
			++(*yScroll);
		}
	}
	else if (player.object.nextPosition.y < player.object.position.y)
	{	
		// Moving up.
		if (player.object.nextPosition.y > 4 && player.object.nextPosition.y < (MAP_TILES_Y - 7))
		{
			--(*yScroll);
		}
	}

	mapRedraw = TRUE;
}

void catchupScroll(u8* xScroll, u8* yScroll, u8* destinationX, u8* destinationY)
{
	u8 i = 0;
	u8 xDifference = 0;
	u8 yDifference = 0;
	i8 xDirection = 0;
	i8 yDirection = 0;

	// Determines how far and what direction the teleport\move took place.
	if (*destinationX > player.object.position.x)
	{
		xDifference = *destinationX - player.object.position.x;
		xDirection = 1;
	}
	else
	{
		xDifference = player.object.position.x - *destinationX;
		xDirection = -1;
	}

	if (*destinationY > player.object.position.y)
	{
		yDifference = *destinationY - player.object.position.y;
		yDirection = 1;
	}
	else
	{
		yDifference = player.object.position.y - *destinationY;
		yDirection = -1;
	}

	// Reset player next position before updating it for the teleport scroll.
	player.object.nextPosition.x = player.object.position.x;
	player.object.nextPosition.y = player.object.position.y;

	// Need to scroll from xPosition to xDestination.
	// We can do this by scrolling the screen one square at a time until we reach the destination.
	for (i = 0; i < xDifference; ++i)
	{
		player.object.nextPosition.x += xDirection;

		scrollScreen(xScroll, yScroll);
	}

	// Reseting this here so that it doesn't try to scroll through x again because scrollScreen does x first and then y.
	player.object.nextPosition.x = player.object.position.x;

	// Need to scroll from yPosition to yDestination.
	// We can do this by scrolling the screen one square at a time until we reach the destination.
	for (i = 0; i < yDifference; ++i)
	{
		player.object.nextPosition.y += yDirection;

		scrollScreen(xScroll, yScroll);
	}

	// Restoring nextPosition.x to the correct vlaue.
	player.object.nextPosition.x = *destinationX;
}

void attackEnemy(u8 x, u8 y, u8 attackType, u8 xScroll, u8 yScroll)
{
	u8 i = 0;

	// Run through the logic for all the enemies.
	for (i = 0; i < mapEnemyCount; i += 3)
	{
		// Find the enemy in the square.
		if (mapEnemies[i] == x && mapEnemies[i + 1] == y)
		{
			u8 cell = MAPA(x, y);

			// Arrows don't effect the dark mine.
			if (attackType == MELEE_ATTACK && (mapEnemies[i + 2] & ENEMY_GFX_RANGE) == ENEMY_DARK_MINE)
			{
				// Player touches a dark mine killing them instantly.
				player.health = 0;

				// Reset the timer.
				timerCount = 0;

				// Indicate there is a death delay going on.
				deathDelay = TRUE;

				// Display the mine explosion where the mine is.
				cell = CELL_LIGHT_FLAG + TILE_MINE_EXPLOSION;

				// Draw the explosion at the mine cell.
				drawCell(x - xScroll, y - yScroll, cell);

				while (deathDelay)
				{
					// Wait for the timer to reset so we can show the player the mine explosion.
				}

				// No need to continue through the loop now the player is dead.
				break;
			}
			else
			{
				u8 enemyHealth = 0;
				u8 meatFlag = 0;

				// On the gesture level enemies are turned into meat.
				if (currentLevel == GESTURE_LEVEL)
				{
					// Meat is 27, item is 32
					meatFlag = 59;
				}

				// Other sorts of enemies follow normal attack rules.
				if (inventoryLeftItem == ITEM_ONE_HANDED_SWORD || inventoryRightItem == ITEM_ONE_HANDED_SWORD || attackType == RANGED_ATTACK)
				{
					// Attack with one handed sword the player will inflict n damage.
					// Ranged attack also does n damage.

					// 192 is the top 2 bits 0b11000000.
					// 63 is the bottom 6 bits.
					// 7 is the bottom 3 bits 0b00000111.
					if (enemyHealth = mapEnemies[i + 2] & 192)
					{
						mapEnemies[i + 2] = (enemyHealth << 1) | (mapEnemies[i + 2] & 63);

						// 128 << 1 is the equivalent of 0.
						if (enemyHealth == 128)
						{
							// Add enemy score to level score on enemy death.
							levelScore += enemyScore(mapEnemies[i + 2] & 7);

							// Enemy is now dead so remove it from the grid and set state to 0.
							mapEnemies[i + 2] = 0;
							
							// Reset cell to empty.
							if (cell & CELL_LIGHT_FLAG)
							{
								// Apply light flag if its already set.
								MAPA(x, y) = CELL_LIGHT_FLAG | meatFlag;
							}
							else
							{
								MAPA(x, y) = meatFlag;
							}

							// Redraw the map to ensure the enemy disappears.
							mapRedraw = TRUE;

							// Redraw hud on score change.
							hudRedraw = TRUE;
						}
					}
				}
				else if (inventoryLeftItem == ITEM_TWO_HANDED_SWORD)
				{
					// Attack with two handed sword the player will inflict 2n damage.

					// 192 is the top 2 bits 0b11000000.
					// 63 is the bottom 6 bits.
					// 7 is the bottom 3 bits 0b00000111.
					if (enemyHealth = mapEnemies[i + 2] & 192)
					{
						mapEnemies[i + 2] = (enemyHealth << 2) | (mapEnemies[i + 2] & 63);

						// 64 << 2 is the equivalent of 0.
						if (enemyHealth == 64)
						{
							// Add enemy score to level score on enemy death.
							levelScore += enemyScore(mapEnemies[i + 2] & 7);

							// Enemy is now dead so remove it from the grid and set state to 0.
							mapEnemies[i + 2] = 0;

							// Reset cell to empty.
							if (cell & CELL_LIGHT_FLAG)
							{
								// Apply light flag if its already set.
								MAPA(x, y) = CELL_LIGHT_FLAG | meatFlag;
							}
							else
							{
								MAPA(x, y) = meatFlag;
							}							

							// Redraw the map to ensure the enemy disappears.
							mapRedraw = TRUE;

							// Redraw hud on score change.
							hudRedraw = TRUE;
						}
					}
				}
				else
				{
					// Attack with no weapon the player will take n damage.
					if (player.health)
					{
						--player.health;
					}
				}
			}
		}
	}
}

void attackBoss(u8 x, u8 y, u8 attackType, u8 xScroll, u8 yScroll)
{
	u8 i = 0;

	// Run through the logic for all the boss cells.
	for (i = 0; i < mapBossCellCount; i += 3)
	{
		// Find the boss in the square.
		if (mapBossCells[i] == x && mapBossCells[i + 1] == y)
		{
			u8 bossGraphic = mapBossCells[i + 2];

			// Gesture boss
			if (bossGraphic >= 9 && bossGraphic <= 12)
			{
				// Player attacks the gesture boss killing the player.
				player.health = 0;

				// Reset the timer.
				timerCount = 0;

				// Indicate there is a death delay going on.
				deathDelay = TRUE;

				// Display the gesture attack graphic where the player is.
				MAPA(player.object.position.x, player.object.position.y) = CELL_LIGHT_FLAG + TILE_GESTURE_ATTACK;

				// Draw the gesture attack at the player cell.
				drawCell(player.object.position.x - xScroll, player.object.position.y - yScroll, MAPA(player.object.position.x, player.object.position.y));

				while (deathDelay)
				{
					// Wait for the timer to reset so we can show the player the mine explosion.
				}

				// Set level end to true as the player has died.
				levelEnded = TRUE;

				// Break out of the loop as the rest of the boss logic does not need to run after the attack.
				break;
			}
			else if (bossGraphic >= 4)
			{
				// TODO Dargon boss.
				u8 cell = MAPA(x, y);
				u8 playerAttackDamage = 0;

				if (inventoryLeftItem == ITEM_ONE_HANDED_SWORD || inventoryRightItem == ITEM_ONE_HANDED_SWORD || attackType == RANGED_ATTACK)
				{
					playerAttackDamage = 1;
				}
				else if (inventoryLeftItem == ITEM_TWO_HANDED_SWORD)
				{
					playerAttackDamage = 2;
				}
				else
				{
					// Attack with no weapon the player will take n damage.
					if (player.health)
					{
						--player.health;
					}
				}

				// Attack the boss for the amount of damage the player can deliever.
				if (finalBossHealth < playerAttackDamage)
				{
					finalBossHealth = 0;
				}
				else
				{
					finalBossHealth -= playerAttackDamage;
				}

				// Having attacked the boss we can now break out of the loop.
				break;
			}
		}
	}
}

void teleporterScroll(u8* xScroll, u8* yScroll)
{
	catchupScroll(xScroll, yScroll, &playerTeleportingX, &playerTeleportingY);

	// Reset teleporter values following scrolling to the new player position.
	playerTeleportingX = 255;
	playerTeleportingY = 255;

	mapRedraw = TRUE;
}

u8 isInteractableReady(u8 cell)
{
	switch(cell)
	{
		// Make sure each door has a matching key.
		case INTERACTABLE_DOOR_RED:
		{
			if(INVENTORY_LINE_1 & INVENTORY_ITEM_KEY_RED)
			{
				// Remove the red key from the inventory on use.
				INVENTORY_LINE_1 &= ~INVENTORY_ITEM_KEY_RED;

				return TRUE;
			}
		}
		break;

		// Make sure each door has a matching key.
		case INTERACTABLE_DOOR_GREEN:
		{
			if(INVENTORY_LINE_1 & INVENTORY_ITEM_KEY_GREEN)
			{
				// Remove the green key from the inventory on use.
				INVENTORY_LINE_1 &= ~INVENTORY_ITEM_KEY_GREEN;

				return TRUE;	
			}
		}
		break;

		// Make sure each door has a matching key.
		case INTERACTABLE_DOOR_YELLOW:
		{
			if(INVENTORY_LINE_1 & INVENTORY_ITEM_KEY_YELLOW)
			{
				// Remove the green key from the inventory on use.
				INVENTORY_LINE_1 &= ~INVENTORY_ITEM_KEY_YELLOW;

				return TRUE;
			}
		}
		break;

		case INTERACTABLE_SWITCH_ON:
		case INTERACTABLE_SWITCH_OFF:
		{
			if(gameControls.useIsDown)
			{
				activateLinkedInteractable(player.object.nextPosition.x, player.object.nextPosition.y, cell);

				return TRUE;
			}
		}
		break;

		case INTERACTABLE_CARD_READER_RED:
		{
			if(INVENTORY_LINE_1 & INVENTORY_ITEM_KEY_CARD_RED)
			{
				activateLinkedInteractable(player.object.nextPosition.x, player.object.nextPosition.y, cell);

				// Remove the red key card from the inventory on use.
				INVENTORY_LINE_1 &= ~INVENTORY_ITEM_KEY_CARD_RED;

				return TRUE;
			}
		}
		break;

		case INTERACTABLE_CARD_READER_GREEN:
		{
			if(INVENTORY_LINE_1 & INVENTORY_ITEM_KEY_CARD_GREEN)
			{
				activateLinkedInteractable(player.object.nextPosition.x, player.object.nextPosition.y, cell);

				// Remove the green key card from the inventory on use.
				INVENTORY_LINE_1 &= ~INVENTORY_ITEM_KEY_CARD_GREEN;

				return TRUE;
			}
		}
		break;

		case INTERACTABLE_CARD_READER_YELLOW:
		{
			if(INVENTORY_LINE_1 & INVENTORY_ITEM_KEY_CARD_YELLOW)
			{
				activateLinkedInteractable(player.object.nextPosition.x, player.object.nextPosition.y, cell);

				// Remove the yellow key card from the inventory on use.
				INVENTORY_LINE_1 &= ~INVENTORY_ITEM_KEY_CARD_YELLOW;

				return TRUE;
			}
		}
		break;

		case INTERACTABLE_TELEPORTER_ON:
		{
			// TODO conditions for teleporting.
			activateLinkedInteractable(player.object.nextPosition.x, player.object.nextPosition.y, cell);

			return TRUE;
		}
		break;

		case INTERACTABLE_BRIDGE_BASE_LEFT:
		case INTERACTABLE_BRIDGE_BASE_RIGHT:
		{
			// Check nails and hammer are being carried.
			if (INVENTORY_LINE_2 & INVENTORY_ITEM_NAIL_BOX && INVENTORY_ITEM_HAMMER & INVENTORY_LINE_1)
			{
				// Check wood block is equiped in one of the hands (doesn't matter which).
				if (inventoryLeftItem == ITEM_WOOD_BLOCK)
				{
					// Remove block from hands.
					inventoryLeftItem = 0;
					inventoryRightItem = 0;

					// Remove block from inventory.
					INVENTORY_LINE_2 &= ~INVENTORY_ITEM_WOOD_BLOCK;

					return TRUE;
				}
			}

			// Conditions not met.
			return FALSE;				
		}
		break;

		case INTERACTABLE_WATER_BLOCKER:
		{
			if (INVENTORY_LINE_1 & INVENTORY_ITEM_GLOVE)
			{
				// Remove the glove from the inventory on use.
				INVENTORY_LINE_1 &= ~INVENTORY_ITEM_GLOVE;

				return TRUE;
			}
		}
		break;

		case INTERACTABLE_GESTURE_PLANT:
		{
			if (player.meat >= plantHunger)
			{
				return INTERACTABLE_GESTURE_PLANT;
			}
		}
		break;

		default:
		{
			// Everything else is true for the moment.
			return TRUE;
		}
		break;
	}

	// Not carrying a matching key so we return false;
	return FALSE;
}

void consumeItem(u8 cell)
{
	switch (cell)
	{
		case ITEM_LAMP_OIL:
			// TODO scale max oil fuel with difficulty.
			// Maximum oil fuel is 25 for now.			
			if (levelOilFuel < 23)
			{
				levelOilFuel += 3;
			}
		break;

		case ITEM_ELECTRIC_TORCH_BATTERY:
			// TODO scale max battery fuel with difficulty.
			// Maximum battery fuel is 25 for now.
			if (levelBatteryFuel < 23)
			{
				levelBatteryFuel += 3;
			}
		break;

		case ITEM_PLACEHOLDER_HEALTH_MEDKIT:
		case ITEM_PLACEHOLDER_HEALTH_APPLE:
			// Add health up to the maximum value.
			if (player.health <= maxHealth - healthItemValue)
			{
				player.health += healthItemValue;
			}
			else
			{
				player.health = maxHealth;
			}
		break;

		case ITEM_ARROW:
			// TODO handle max arrows if necessary.
			levelArrowCount += 5;
		break;

		case ITEM_POTION:
			// Increases the maximum dark move count on pickup for the level session.
			maxDarkMoves += 6;
			darkMoveCount += 6;
		break;

		case ITEM_MEAT:
			++player.meat;
		break;
	}
}